(function($) {
    jQuery.fn.toc = function(options) {

        /* GET DEFAULT OPTIONS OR USE THE ONE PASSED IN THE FUNCTION  */
        var opts      = $.extend({}, $.fn.toc.defaults, options),
            n         = 1
            root      = $("<"+opts.listType+">").addClass(opts.listClass),
            parent    = root,
            currlevel = 1;

        $(opts.selector).each(function() {

            var item  = $(this),
                title = item.text(),
                link  = "#" + item.attr("id");

            /* GET HEADER LEVEL */
            var level = item.attr("aria-level");
            if(typeof level == "undefined") {
                var name = item[0].tagName;
                level = name.replace(/h/gi,"");
            }

            /* MAKE SURE WE HAVE A LINK */
            if(link=='#undefined') {
                item.attr('id','toc_'+n);
                link = "#toc_"+n;
                n++;
            }

            /* BUILD THE ANCHOR ITEM */
            var a = $('<a/>')
                .text(title)
                .attr('href', link)
                .bind('click', function(e) {
                    
                });

            /* BUILD THE LIST ITEM */
            var li = $('<li/>')
                .addClass(opts.itemClass)
                .addClass(opts.itemClass + "_l" + level)
                .append(a);

            /* FIND PARENT */
            switch(true) {
                case (level > currlevel):
                    currlevel = level;
                    var node = $("<"+opts.listType+">");
                    parent.find('li:last-of-type').append(node); // find last li
                    parent = parent.find('li:last-of-type > '+opts.listType);
                    break;
                case (level < currlevel):
                    var steps = currlevel-level;
                    parent = parent.first().parents().eq(steps);
                    currlevel = level;
                    break;
                default:
                    break;
            } // switch

            /* APPEND THE NODE */
            parent.append(li);

        }); // $(opts.selector).each()

        /* PARSE TEMPLATE */
        var output = opts.template
             .replace(/{{header_text}}/gi,opts.header_text)
             .replace(/{{navClass}}/gi,opts.navClass)
             .replace(/{{toc}}/gi, $(root)[0].outerHTML);

        $(output).appendTo($(this));



    }; // function jQuery.fn.toc()

    $.fn.toc.defaults = {
        header_text   : "On this page:",
        navClass      : "toc",
        listClass     : "toc",
        itemClass     : "toc_item",
        listType      : "ul",
        selector      : "h1, h2, h3",
        template      : "<nav role='navigation' class='{{navClass}}'>" +
                        "<strong role='heading'>{{header_text}}</strong>" +
                        "{{toc}}" +
                        "</nav>"
    };
})(jQuery);
